@@ -1,10 +1,28 @@
+# $Id: Error.pm,v 1.1.2.1 2004/04/20 20:09:48 pajas Exp $
+#
+# This is free software, you may use it and distribute it under the same terms as
+# Perl itself.
+#
+# Copyright 2001-2003 AxKit.com Ltd., 2002-2006 Christian Glahn, 2006-2009 Petr Pajas
+#
+#
 package XML::LibXML::Error;
 
 use strict;
-use vars qw($AUTOLOAD @error_domains);
+use vars qw($AUTOLOAD @error_domains $VERSION $WARNINGS);
 use Carp;
-use overload
-  '""' => \&as_string;
+use overload 
+  '""' => \&as_string,
+  'eq' => sub {
+    ("$_[0]" eq "$_[1]")
+  },
+  'cmp' => sub {
+    ("$_[0]" cmp "$_[1]")
+  },
+  fallback => 1;
+
+$WARNINGS = 0; # 0: supress, 1: report via warn, 2: report via die
+$VERSION = "1.70"; # VERSION TEMPLATE: DO NOT CHANGE
 
 use constant XML_ERR_NONE	     => 0;
 use constant XML_ERR_WARNING	     => 1; # A simple warning
@@ -49,6 +67,7 @@ use constant XML_ERR_FROM_VALID	     => 23; # The validaton module
     my ($class,$xE) = @_;
     my $terr;
     if (ref($xE)) {
+      my ($context,$column) = $xE->context_and_column();
       $terr =bless {
 	domain  => $xE->domain(),
 	level   => $xE->level(),
@@ -61,6 +80,11 @@ use constant XML_ERR_FROM_VALID	     => 23; # The validaton module
 	str3    => $xE->str3(),
 	num1    => $xE->num1(),
 	num2    => $xE->num2(),
+	(defined($context) ?
+	   (
+	     context => $context,
+	     column => $column,
+	    ) : ()),
       }, $class;
     } else {
       # !!!! problem : got a flat error
@@ -84,14 +108,18 @@ use constant XML_ERR_FROM_VALID	     => 23; # The validaton module
 
     sub _callback_error {
       #print "CALLBACK\n";
-      my ($xE,$prev) = shift;
+      my ($xE,$prev) = @_;
       my $terr;
       $terr=XML::LibXML::Error->new($xE);
-      unless ( defined $terr->{file} and length $terr->{file} ) {
+      if ($terr->{level} == XML_ERR_WARNING and $WARNINGS!=2) {
+	warn $terr if $WARNINGS;
+	return $prev;
+      }
+      #unless ( defined $terr->{file} and length $terr->{file} ) {
 	# this would make it easier to recognize parsed strings
 	# but it breaks old implementations
 	# [CG] $terr->{file} = 'string()';
-      }
+      #}
       #warn "Saving the error ",$terr->dump;
       $terr->{_prev} = ref($prev) ? $prev :
 	defined($prev) && length($prev) ? XML::LibXML::Error->new($prev) : undef;
@@ -115,8 +143,6 @@ use constant XML_ERR_FROM_VALID	     => 23; # The validaton module
       my ($saved_error) = @_;
       #print "CALLBACK ERROR: $saved_error\n";
       if ( defined $saved_error ) {
-	use Data::Dumper;
-	# print "reporting error ",Dumper($saved_error);
 	die $saved_error;
       }
     }
@@ -128,13 +154,17 @@ sub AUTOLOAD {
   return undef unless ref($self);
   my $sub = $AUTOLOAD;
   $sub =~ s/.*:://;
-  if ($sub=~/^(?:code|_prev|level|file|line|domain|nodename|message|num[123]|num[12])$/) {
+  if ($sub=~/^(?:code|_prev|level|file|line|domain|nodename|message|column|context|str[123]|num[12])$/) {
     return $self->{$sub};
   } else {
     croak("Unknown error field $sub");
   }
 }
 
+# backward compatibility
+sub int1 { $_[0]->num1 }
+sub int2 { $_[0]->num2 }
+
 sub DESTROY {}
 
 sub domain {
@@ -177,9 +207,15 @@ sub as_string {
     chomp($str);
     $msg.=" ".$str."\n";
     if (($self->{domain} == XML_ERR_FROM_XPATH) and
-        defined($self->{str1})) {
-        $msg.=$self->{str1}."\n";
-        $msg.=(" " x $self->{num1})."^\n";
+	  defined($self->{str1})) {
+      $msg.=$self->{str1}."\n";
+      $msg.=(" " x $self->{num1})."^\n";
+    } elsif (defined $self->{context}) {
+      my $context = $self->{context};
+      $msg.=$context."\n";
+      $context = substr($context,0,$self->{column});
+      $context=~s/[^\t]/ /g;
+      $msg.=$context."^\n";
     }
     return $msg;
 }
