@@ -1,4 +1,11 @@
-# $Id: Builder.pm 741 2008-11-02 22:38:48Z pajas $
+# $Id: Builder.pm 785 2009-07-16 14:17:46Z pajas $
+#
+# This is free software, you may use it and distribute it under the same terms as
+# Perl itself.
+#
+# Copyright 2001-2003 AxKit.com Ltd., 2002-2006 Christian Glahn, 2006-2009 Petr Pajas
+#
+#
 
 package XML::LibXML::SAX::Builder;
 
@@ -11,7 +18,7 @@ sub CLONE_SKIP {
   return $XML::LibXML::__threads_shared ? 0 : 1;
 }
 
-$VERSION = "1.68"; # VERSION TEMPLATE: DO NOT CHANGE
+$VERSION = "1.70"; # VERSION TEMPLATE: DO NOT CHANGE
 
 sub new {
     my $class = shift;
@@ -50,7 +57,6 @@ sub end_dtd {
 
 sub start_document {
     my ($self, $doc) = @_;
-    print "start document called\n";
     $self->{DOM} = XML::LibXML::Document->createDocument();
 
     if ( defined $self->{Encoding} ) {
@@ -78,7 +84,6 @@ sub xml_decl {
 
 sub end_document {
     my ($self, $doc) = @_;
-    print "end_document called\n";
     my $d = $self->done();
     return $d;
 }
